/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is JIM.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * services.IQ.Last
 *
 *
 */
ML.load("utils/observable.js");
ML.load("utils/misc.js");

/**
 * IQLast
 *   jabber:iq:last protocol handler
 *
 * Constructors:
 *   IQLast(jabberService) - constructor
 *
 * Methods:
 *   getLastActivity(from)
 *
 * Events:
 *   onIQLastError(errorCode, errorText)
 *   onIQLast(from, seconds, status)
 */

function IQLast(jabberService)
{
    this.base = Observable;
    this.base();

    this.jabberService = jabberService;
    this.jabberService.core.iq.disco.registerFeature("jabber:iq:last");
}

IQLast.prototype =
{
    __proto__: Observable.prototype,
    namespaceURI: "jabber:iq:last",

    getLastActivity: function(from)
    {
        this.jabberService.core.iqTag(null, from, null, "get", 
            DB.createElement("jabber:iq:last", "query"), this);
    },

    onIQResponse: function(tag)
    {
        if (tag.getAttribute("type")=="error") {
            var errTag = tag.getElementsByTagName("error")[0];

            this.fireEvent("onIQLastError", errTag.getAttribute("code"),
                domToText(errTag));
        } else if (tag.getAttribute("type")=="result") {
            var query = tag.getElementsByTagNameNS(this.namespaceURI,
                "query")[0];
            var seconds = query && query.getAttribute("seconds");
            
            this.fireEvent("onIQLast", tag.getAttribute("from"),
                seconds, query && domToText(query));
        }
    },

    onIQHandler: function(tag)
    {
        var now, query;

        if (tag.getAttribute("type") != "get")
            return;
        
        if (!getBoolPref("proto-last-send")) {
            this.iqError(tag.getAttribute("to"),
                tag.getAttribute("from"),
                tag.getAttribute("id"),
                501,
                "Not Implemented");
            return;
        }
        
        now = new Date();

        query = DB.createElement("jabber:iq:last", "query",
				 "seconds", ((now.getTime() -
					      this.jabberService.
					      connectionTime.getTime()
					      )/1000).toFixed(0));

        this.jabberService.core.iqTag(this.jabberService.core.uid, tag.getAttribute("from"),
            tag.getAttribute("id"), "result", query);
    },
}
