ML.load("utils/misc.js");

function __registerHandler(obj, prop, handler)
{
    var getter;

    if (getter = obj.__lookupGetter__(prop)) {
	getter(handler);
    } else {
	var handlers = [handler];
	var value = obj[prop];

	obj[prop] getter = function(handler, remove) {
	    var i;

	    if (handler) {
		if (remove) {
		    for (i = 0; i < handlers.length; i++)
			if (handlers[i] == handler)
			    break;
		    if (i < handlers.length)
			handlers.splice(i, 1);
		} else
		    handlers.push(handler);
	    }
	    return value;
	};

	obj[prop] setter = function(val) {
	    var i, oldVal;

	    if (val != value) {
		oldVal = value;
		value = val;

		for (i = 0; i < handlers.length; i++)
		    handlers[i].handler(obj, prop, oldVal, value);
	    }
	    return val;
	};
    }
}

function __unregisterHandler(obj, prop, handler) {
    var getter;

    if (getter = obj.__lookupGetter__(prop))
	getter(handler, true);
}

function ExprWatcher(expr, handler, global, bindNames, bindValues,
		     dontGenerateInitialEvents, bindHash)
{
    var i, tmp;
    var tree = [], exprs = [];
    var exprRes;

    this.ex = expr;

    this.handlerFn = handler;
    this.global = global;
    this.bindValues = bindValues;

    if (bindHash) {
	this.bindHash = bindHash;
	this.bindNames = bindNames;
    } else
	this.__genBindData(bindNames);

    this.bindNames.push("try{return "+expr+"}catch(e){return null;}");
    this.expr = Function.apply(null, this.bindNames);
    this.bindNames.pop();
	    

    exprRes = grep(function(a){return !!a}, expr.split(this.splitRE));
    this.__buildTree(exprRes, 0, tree);
    this.__findSubExpr(tree, exprs);

    exprs = exprs.sort(function(a,b){
	return a[0].length>b[0].length||a[0]>b[0] ? 1:a[0]==b[0]?0:-1;
    });

    this.exprs = [exprs[0]];
    for (i = 0; i < exprs.length-1; i++)
	if (exprs[i][0] != exprs[i+1][0])
	    this.exprs.push(exprs[i+1]);

    for (i = 0; i < this.exprs.length; i++) {
	this.bindNames.push("try{return "+this.exprs[i][0]+"}catch(e){return null;}");
	this.exprs[i][0] = Function.apply(null, this.bindNames);
	this.bindNames.pop();
	this.bindNames.push("try{return "+this.exprs[i][1]+"}catch(e){return null;}");
	this.exprs[i][1] = Function.apply(null, this.bindNames);
	this.bindNames.pop();
    }
    if (!this.bindNames || this.bindValues)
	this.handler(!dontGenerateInitialEvents);
}

ExprWatcher.prototype =
{
    idRE: new RegExp("^[a-zA-Z]\\w*$"),
    splitRE: new RegExp("\\.|(\\[|\\]|\\(|\\)|\"|'|\\W)"),

    bind: function bind(bindValues, dontGenerateInitialEvents) {
	this.bindValues = bindValues;
	this.handler(!dontGenerateInitialEvents);
    },

    destroy: function() {
	for (i = 0; i < this.exprs.length; i++)
	    if (this.exprs[i].obj && this.exprs[i].prop)
		__unregisterHandler(this.exprs[i].obj, this.exprs[i].prop, this);
    },

    __genBindData: function(bindNames)
    {
	var i, tmp;

	this.bindNames = [];
	this.bindHash = {};

	if (bindNames) {
	    for (i = 0; i < bindNames.length; i++)
		if ((tmp = bindNames[i]) instanceof Array) {
		    this.bindHash[tmp[0]] =
			[tmp[1], "'"+tmp[2]+"'"];
		    this.bindNames.push(tmp[0]);
		} else
		    this.bindNames.push(tmp);
	}
    },

    __buildTree: function(input, idx, result) {
	var tmp, ch;

	while (idx < input.length)
	    switch (ch = input[idx]) {
	    case "[":
	    case ")":
		result.push(tmp = []);
		tmp.type = ch;
		idx = this.__buildTree(input, ++idx, tmp);
		break;
	    case "\"":
	    case "'":
		tmp = ch;
		while (++idx < input.length && input[idx] != ch)
		    if (input[idx])
			tmp += input[idx];
		idx++;
		result.push(tmp+ch);
		break;
	    case "]":
	    case ")":
		return ++idx;
	    default :
		result.push(input[idx++]);
	    }
	return ++idx;
    },

    __findSubExpr: function(tree, result, startIdx)
    {
	var i, currResult = "", newExpr = true, temp;
	startIdx = startIdx || 0;
	
	for (i = startIdx; i < tree.length; i++) {
	    if (typeof(tree[i]) == "object") {
		if (tree[i].type == "(") {
		    currResult+="("+this.__findSubExpr(tree[i], result)+")";
		} else {
		    tmp = this.__findSubExpr(tree[i], result)
		    result.push([currResult, tmp]);
		    currResult+="["+tmp+"]";
		}
	    } else if (tree[i].match(this.idRE)) {
		if (newExpr)
		    if (i != startIdx)
			currResult += this.__findSubExpr(tree, result, i);
		    else {
			currResult += tree[i];
			tmp = this.bindHash[tree[i]];
			if (!tmp || tmp[0] != "" || tmp[1] != "")
			    result.push([(tmp && tmp[0]) || "this",
					 (tmp && tmp[1]) || "'"+tree[i]+"'"]);
		    }
		else {
		    result.push([currResult, "'"+tree[i]+"'"]);
		    currResult += "."+tree[i];
		}
		newExpr = false;
	    } else {
		newExpr = true;
		currResult += tree[i];
	    }
	}
	return currResult;
    },

    handler: function handler(_obj, _prop, _oldval, _newval) {
	var obj, prop, value, i;

	for (i = 0; i < this.exprs.length; i++) {
	    obj = this.exprs[i][0].apply(this.global, this.bindValues);
	    prop = this.exprs[i][1].apply(this.global, this.bindValues);

	    if (obj != this.exprs[i].obj || prop != this.exprs[i].prop) {
		if (this.exprs[i].obj && this.exprs[i].prop)
		    __unregisterHandler(this.exprs[i].obj, this.exprs[i].prop, this)
		this.exprs[i].obj = obj;
		this.exprs[i].prop = prop;
		if (obj && prop)
		    __registerHandler(obj, prop, this)
	    }
	}

	if ((value = this.expr.apply(this.global, this.bindValues)) != this.value) {
	    this.value = value;
	    if (_obj)
		this.handlerFn.handler(this, value);
	}
    },
}

function MultiExprWatcher(expr, handler, global, bindNames, bindValues,
			  dontGenerateInitialEvents)
{
	var vals, i, ex, exprRes;

	this.exprWatchers = [];
	this.handlerFn = handler;
	this.bindValues = bindValues;
	this.global = global;

	this.__genBindData(bindNames);

	vals = expr.split(this.exprRE);

	exprRes = "";

	for (i = 0; i < vals.length; i++)
	    if (ex = this.subExprRE.exec(vals[i])) {
		ex = ex[1];
		this.exprWatchers.
		    push(new ExprWatcher(ex, this, global,
					 this.bindNames, bindValues,
					 true, this.bindHash));
		exprRes+="if((__subVAL__="+
		    ex+
		    ")!=null)__retVAL__+=__subVAL__;else return null;"
	    } else if (vals[i])
		exprRes += "__retVAL__+=\""+vals[i].replace(/\"/g,"\\\"").
		    replace(/\$\$/g, "$")+"\";";

	this.bindNames.push("try {var __subVAL__,__retVAL__='';"+exprRes+
			    "return __retVAL__}catch(ex){return null}");
	this.expr = Function.apply(null, this.bindNames);
	this.bindNames.pop();

	if (!bindNames || bindValues)
	    this.handler(!dontGenerateInitialEvents);
}

MultiExprWatcher.prototype = {
    exprRE: new RegExp("(\\$\\$|\\$\\{.*?\\})", "g"),
    subExprRE: new RegExp("^\\$\\{(.*)\\}$"),
    varsDivRE: new RegExp("^(.*)\\.([^\\.]*)$"),

    __genBindData: ExprWatcher.prototype.__genBindData,

    bind: function bind(bindValues, dontGenerateInitialEvents) {
	this.bindValues = bindValues;
	for (i = 0; i < this.exprWatchers.length; i++)
	    this.exprWatchers[i].bind(bindValues, true);

	this.handler(!dontGenerateInitialEvents);
    },

    destroy: function(dontRemoveElements) {
	var i;

	for (i = 0; i < this.exprWatchers.length; i++)
	    this.exprWatchers[i].destroy();
    },

    handler: function handler(generateInitialEvents)
    {
	var value;

	if ((value = this.expr.apply(this.global, this.bindValues)) != this.value) {
	    this.value = value;
	    if (generateInitialEvents)
		this.handlerFn.handler(this, value);
	}
    },
}
