ML.load("utils/exprwatcher.js");

function __BaseTemplate(previousTemplate, parentObject, parentProp)
{
    this.previousTemplate = previousTemplate;
    this.parentObject = parentObject;
    this.parentProp = parentProp;
}

__BaseTemplate.prototype =
{
    getBuilderForTag: function(tag, previousTemplate, parentObject, parentProp)
    {
	const TMPL_NS="http://jim.mozdev.org/NS/templates";
	
	if (tag.nodeType == tag.TEXT_NODE)
	    return new __TextTemplate(tag, previousTemplate,
				    parentObject, parentProp);
	else if (tag.nodeType == tag.ELEMENT_NODE) 
	    if (tag.namespaceURI == TMPL_NS)
		switch (tag.localName) {
		case "value-of":
		    return new __ValueOfTemplate(tag, previousTemplate,
					       parentObject, parentProp);
		case "choose":
		    return new __ChooseTemplate(tag, previousTemplate,
					      parentObject, parentProp);
		case "for-each":
		    return new __ForEachTemplate(tag, previousTemplate,
					      parentObject, parentProp);
		}
	    else
		return new __TagTemplate(tag, previousTemplate,
				       parentObject, parentProp);
    },

    createWatcher: function(expression, parentObject, parentProp)
    {
	if (parentObject == null)
	    parentObject = this.parentObject;
	if (parentProp == null)
	    parentProp = this.parentProp;

	if (parentObject && parentProp)
	    return new ExprWatcher(expression, this, window,
				   [["value", parentObject, parentProp]],
				   null, false);
	else
	    return new ExprWatcher(expression, this, window,
				   ["value"], null, false);
    },

    createSpecWatcher: function(expression, parentObject, parentProp)
    {
	if (parentObject == null)
	    parentObject = this.parentObject;
	if (parentProp == null)
	    parentProp = this.parentProp;

	if (parentObject && parentProp)
	    return new MultiExprWatcher(expression, this, window,
				   [["value", parentObject, parentProp]],
				   null, false);
	else
	    return new MultiExprWatcher(expression, this, window,
				   ["value"], null, false);
    },

    valueChanged: function valueChanged(value)
    {
	this.expr.bind([value]);
    },
}

function __Template(tag)
{
    var ctag, ctag2;

    this.childNodes = [];
    this.tag = tag;

    tag = tag.firstChild;
    ctag = this;

    while (tag) {
	if (ctag2 = this.getBuilderForTag(tag, ctag, "", "")) {
	    this.childNodes.push(ctag = ctag2);
	}
	tag = tag.nextSibling;
    }
}

__Template.prototype =
{
    __proto__: __BaseTemplate.prototype,

    bind: function create(value)
    {
	var i;

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].create(value, this.tag.parentNode);
    },

    destroy: function(dontRemoveElements)
    {
	var i;

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].destroy();
    },

    get previousSibling()
    {
	return this.tag.previousSibling;
    },

    get nextSibling()
    {
	return this.tag.nextSibling;
    },

    gatherResults: function()
    {
	var i;

	for (i = 0; i < this.childNodes; i++)
	    this.childNodes.gatherResults();
    }
}


function __ValueOfTemplate(tag, previousTemplate, parentObject, parentProp)
{
    this.base = __BaseTemplate;
    this.base(previousTemplate, parentObject, parentProp);

    this.expr = this.createWatcher(tag.getAttribute("select"),
				   parentObject, parentProp);
}

__ValueOfTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    create: function create(value, domRoot)
    {
	this.value = value;
	this.domRoot = domRoot;

	this.firstNode = this.lastNode = 
	    this.node = document.createTextNode("");

	domRoot.insertBefore(this.node,
			     this.previousTemplate ?
			     this.previousTemplate.nextSibling :
			     domRoot.firstChild);


	this.valueChanged(value);
    },

    handler: function handler()
    {
	if (this.expr.value != null)
	    this.node.replaceData(0, this.node.length, this.expr.value);
	else
	    this.node.replaceData(0, this.node.length, "");
    },

    destroy: function destroy(dontRemoveElements)
    {
	this.expr.destroy();

	if (!dontRemoveElements) {
	    this.domRoot.removeChild(this.node);
	    this.firstNode = this.lastNode = null;
	}
    },

    get previousSibling()
    {
	return this.node.previousSibling;
    },

    get nextSibling()
    {
	return this.node.nextSibling;
    },

    gatherResults: function gatherResults()
    {
    }
}

function __ChooseTemplate(tag, previousTemplate, parentObject, parentProp)
{
    var i, ctag, test, condition, work = true;

    const TMPL_NS="http://jim.mozdev.org/NS/templates";

    this.base = __BaseTemplate;
    this.base(previousTemplate, parentObject, parentProp);

    this.conditions = [];

    for (i = 0; work && i < tag.childNodes.length; i++)
	if ((ctag = tag.childNodes[i]).nodeType == tag.ELEMENT_NODE &&
	    ctag.namespaceURI == TMPL_NS)
	    {
		if(ctag.localName == "when") {
		    if (test = ctag.getAttribute("test"))
			condition = {
			    watcher: this.createWatcher(test, parentObject,
							parentProp),
			    childNodes: []
			}
		} else if (ctag.localName == "otherwise") {
		    condition = {
			watcher: {value: true, bind: function(){}, destroy: function(){}},
			childNodes: []
		    }
		    work = false;
		}
		ctag = previousTemplate;
		for (j = 0; j < ctag.childNodes.length; j++)
		    if (test = this.getBuilderForTag(ctag.childNodes[j], ctag,
						     parentObject, parentProp))
			condition.childNodes.push(ctag = test);
		this.conditions.push(condition);
	    }
    this.activeCondition = this.conditions.length;
}

__ChooseTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    get firstNode()
    {
	var child, i;

	if (this.activeCondition < this.conditions.length) {
	    child = this.conditions[this.activeCondition].childNodes;

	    for (i = 0; i < child.length; i++)
		if (child[i].firstNode)
		    return child[i].firstNode;
	}
	return null;
    },

    get lastNode()
    {
	var child, i;

	if (this.activeCondition < this.conditions.length) {
	    child = this.conditions[this.activeCondition].childNodes;

	    for (i = child.length-1; i >= 0; i--)
		if (child[i].lastNode)
		    return child[i].lastNode;
	}
	return null;
    },

    create: function create(value, domRoot)
    {
	var val, i;

	this.value = value;
	this.domRoot = domRoot;

	this.valueChanged(value);
    },

    valueChanged: function valueChanged(value)
    {
	for (i = 0; i < this.conditions.length; i++)
	    this.conditions[i].watcher.bind([value], true);

	this.handler()
    },

    handler: function handler()
    {
	var i, j;
	var condition, sibling;

	for (i = 0; i < this.conditions.length; i++)
	    if (this.conditions[i].watcher.value)
		break;

	if (i != this.activeCondition) {
	    if (this.activeCondition < this.conditions.length) {
		condition = this.conditions[this.activeCondition];
		for (j = 0; j < condition.childNodes.length; j++)
		    condition.childNodes[j].destroy(true);
	    }

	    for (j = this.firstNode; j; j = sibling) {
		sibling = j.nextSibling;
		this.domRoot.removeChild(j);

		if (j == this.lastNode)
		    break;
	    }

	    this.activeCondition = i;

	    if (i < this.conditions.length) {
		condition = this.conditions[i];
		for (j = 0; j < condition.childNodes.length; j++)
		    sibling = condition.childNodes[j].create(this.value,
							     this.domRoot);
	    }
	}
    },

    destroy: function destroy(dontRemoveElements)
    {
	var i, condition;

	if (this.activeCondition < this.conditions.length) {
	    condition = this.conditions[this.activeCondition];
	    for (i = 0; i < condition.childNodes.length; i++)
		condition.childNodes[i].destroy(true);
	}

	if (dontRemoveElements)
	    for (i = this.firstNode; i; i = sibling) {
		sibling = i.nextSibling;
		this.domRoot.removeChild(i);

		if (i == this.lastNode)
		    break;
	    }
    },

    get previousSibling()
    {
	return this.firstNode && this.firstNode.previousSibling;
    },

    get nextSibling()
    {
	return this.lastNode && this.lastNode.nextSibling;
    },

    gatherResults: function gatherResults()
    {
    }
}

function __ForEachTemplate(tag, previousTemplate, parentObject, parentProp)
{
    var i;

    this.base = __BaseTemplate;
    this.base(previousTemplate, parentObject, parentProp);

    this.nodes = [];
    this.nodesHash = {};
    this.tag = tag;
    this.exprTxt = tag.getAttribute("select");

    this.expr = this.createWatcher(this.exprTxt, parentObject, parentProp);
}

__ForEachTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    get firstNode()
    {
	var i, j;

	for (i = 0; i < this.nodes.length; i++)
	    for (j = 0; j < this.nodes[i].length; j++)
		if (this.nodes[i][j].firstNode)
		    return this.nodes[i][j].firstNode
	return null;
    },

    get lastNode()
    {
	var i, j;

	for (i = this.nodes.length - 1; i >= 0; i--)
	    for (j = this.nodes[i].length - 1; j >= 0; j--)
		if (this.nodes[i][j].lastNode)
		    return this.nodes[i][j].lastNode
	return null;
    },

    create: function create(value, domRoot)
    {
	var val, i;

	this.value = value;
	this.domRoot = domRoot;

	this.valueChanged(value);
    },

    handler: function handler(obj, prop, oldVal, curVal)
    {
	var i, j, val;
	var nodes, pt, pt2;
	var thisObj = this;

	val = this.expr.value;

	if (obj && obj == val && this.nodesHash[prop] != null) {
	    nodes = this.nodes[this.nodesHash[prop]];
	    for (i = 0; i < nodes.length; i++)
		nodes[i].valueChanged(curVal);
	    return;
	}

	if (!val || typeof(val) != "object") {
	    for (i = 0; i < this.nodes.length; i++) {
		for (j = 0; j < this.nodes[i].length; j++)
		    this.nodes[i][j].destroy();
	    }
	    for (i in this.oldVal)
		__unregisterHandler(this.oldVal, i);
	    this.nodes = [];
	    this.nodesHash = {};

	    clearInterval(this.interval);
	    this.interval = null;
	    return;
	}

	this.oldVal = val;

	if (!this.interval)
	    this.interval = setInterval(function(){thisObj.handler()}, 1000);

	for (i in this.nodesHash)
	    if (!val[i] || typeof(val[i]) == "function") {
		nodes = this.nodes[this.nodesHash[i]];
		for (j = 0; j < nodes.length; j++ )
		    nodes[j].destroy();
		this.nodes.splice(this.nodesHash[i], 1);
		__unregisterHandler(val, i, this);
		delete this.nodesHash[i];
	    }

	for (i in val)
	    if (this.nodesHash[i] == null && typeof(val[i]) != "function") {
		pt2 = this.previousTemplate;
		nodes = [];

		__registerHandler(val, i, this);

		for (j = 0; j < this.tag.childNodes.length; j++)
		    if (pt = this.getBuilderForTag(this.tag.childNodes[j],
						   pt2, this.exprTxt, i)) {
			nodes.push(pt2 = pt);
			pt.create(val[i], this.domRoot);
		    }

		this.nodesHash[i] = this.nodes.push(nodes)-1;
	    }
    },

    destroy: function destroy(dontRemoveElements)
    {
	var i,j;

	if (this.interval)
	    clearInterval(this.interval);

	for (i in this.oldVal)
	    __unregisterHandler(this.oldVal, i, this);

	for (i = 0; i < this.nodes.length; i++) {
	    for (j = 0; j < this.nodes[i].length; j++)
		this.nodes[i][j].destroy(dontRemoveElements);
	}

	this.nodes = [];
	this.nodesHash = {};
    },

    get previousSibling()
    {
	return this.firstNode && this.firstNode.previousSibling;
    },

    get nextSibling()
    {
	return this.lastNode && this.lastNode.nextSibling;
    },

    gatherResults: function gatherResults()
    {
    }
}

function __TextTemplate(tag, previousTemplate, parentObject, parentProp)
{
    this.nodeValue = tag.nodeValue;
    this.previousTemplate = previousTemplate;
}

__TextTemplate.prototype =
{
    create: function create(value, domRoot)
    {
	this.domRoot = domRoot;

	this.firstNode = this.lastNode = 
	    this.node = document.createTextNode(this.nodeValue);

	domRoot.insertBefore(this.node,
			     this.previousTemplate ?
			     this.previousTemplate.nextSibling :
			     domRoot.firstChild);
    },

    valueChanged: function()
    {
    },

    destroy: function(dontRemoveElements)
    {
	if (!dontRemoveElements)
	    this.domRoot.removeChild(this.node);
    },

    get previousSibling()
    {
	return this.node.previousSibling;
    },

    get nextSibling()
    {
	return this.node.nextSibling;
    },

    gatherResults: function()
    {
    }
}

function __TagTemplate(tag, previousTemplate, parentObject, parentProp)
{
    var i, ctag;

    this.base = __BaseTemplate;
    this.base(previousTemplate, parentObject, parentProp);

    this.nodeName = tag.nodeName;
    this.namespaceURI = tag.namespaceURI;
    this.childNodes = [];
    this.attributes = [];

    for (i = 0; i < tag.attributes.length; i++)
	if (this.hasExprRE.test(tag.attributes[i].nodeValue))
	    this.childNodes.
		push(new __AttributeTemplate(tag.attributes[i],
					   previousTemplate,
					   parentObject,
					   parentProp));
	else
	    with (tag.attributes[i])
		this.attributes.push([namespaceURI, nodeName, nodeValue]);

    tag = tag.firstChild;
    ctag = null;

    while (tag) {
	if (i = this.getBuilderForTag(tag, ctag, parentObject, parentProp)) {
	    this.childNodes.push(ctag = i);
	}
	tag = tag.nextSibling;
    }
}

__TagTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,
    hasExprRE: new RegExp("^(?:.*[^$])?\\$\\{"),

    create: function create(value, domRoot)
    {
	var i;

	this.domRoot = domRoot;

	this.firstNode = this.lastNode = 
	    this.node = document.
	    createElementNS(this.namespaceURI, this.nodeName);

	for (i = 0; i < this.attributes.length; i++)
	    this.node.setAttributeNS(this.attributes[i][0],
				     this.attributes[i][1],
				     this.attributes[i][2]);

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].create(value, this.node);

	domRoot.insertBefore(this.node,
			     this.previousTemplate ?
			     this.previousTemplate.nextSibling :
			     domRoot.firstChild);
    },

    valueChanged: function valueChanged(value)
    {
	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].valueChanged(value)
    },

    destroy: function(dontRemoveElements)
    {
	var i;

	for (i = 0; i < this.childNodes.length; i++)
	    this.childNodes[i].destroy(true);

	if (!dontRemoveElements)
	    this.domRoot.removeChild(this.node);
    },

    get previousSibling()
    {
	return this.node.previousSibling;
    },

    get nextSibling()
    {
	return this.node.nextSibling;
    },

    gatherResults: function()
    {
	var i;

	for (i = 0; i < this.childNodes; i++)
	    this.childNodes.gatherResults();
    }
}

function __AttributeTemplate(tag, previousTemplate, parentObject, parentProp)
{
    this.base = __BaseTemplate;
    this.base(previousTemplate, parentObject, parentProp);

    this.namespaceURI = tag.namespaceURI;
    this.nodeName = tag.nodeName;

    this.expr = this.createSpecWatcher(tag.nodeValue);
}

__AttributeTemplate.prototype =
{
    __proto__: __BaseTemplate.prototype,

    create: function create(value, domRoot)
    {
	this.domRoot = domRoot;
	this.firstNode = this.lastNode = null;

	this.valueChanged(value);
    },

    handler: function handler()
    {
	if (this.expr.value != null)
	    this.domRoot.setAttributeNS(this.namespaceURI,
					this.nodeName,
					this.expr.value);
	else
	    this.domRoot.removeAttributeNS(this.namespaceURI,
					   this.nodeName);
    },

    destroy: function destroy(dontRemoveElements)
    {
	this.expr.destroy();

	if (!dontRemoveElements)
	    this.domRoot.removeAttributeNS(this.namespaceURI,
					   this.nodeName);
    },

    get previousSibling()
    {
	return this.previousTemplate.previousSibling;
    },

    get nextSibling()
    {
	return this.previousTemplate.nextSibling;
    },

    gatherResults: function gatherResults()
    {
	if (this.expr.lvalue)
	    this.expr.set(this.rootEl.getAttribute(this.nodeName));
    }
}

function bindValueToTemplate(value, dom) {
    var template;

    const TMPL_NS="http://jim.mozdev.org/NS/templates";

    if (!dom || dom.namespaceURI != TMPL_NS || dom.localName != "template")
	return;

    template = new __Template(dom);
    template.bind(value);

    return template;
}
